@@ -343,12 +343,16 @@ const QStringList& SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAu
     QString fileNameIn2;
     QString fileNameOut2;
 
-    if(m_fileAccess.isValid() && !m_fileAccess.isNormal())
+    if(!m_fileAccess.isValid())
+        return mErrors;
+
+    if(!m_fileAccess.isNormal())
     {
         mErrors.append(i18n("%1 is not a normal file.", m_fileAccess.prettyAbsPath()));
         return mErrors;
     }
 
+    Q_ASSERT(!m_fileAccess.isDir());
     bool bTempFileFromClipboard = !m_fileAccess.isValid();
 
     // Detect the input for the preprocessing operations
