@@ -23,6 +23,7 @@
 #include "difftextwindow.h"
 #include "mergeresultwindow.h"
 #include "RLPainter.h"
+#include "Utils.h"
 
 #ifndef Q_OS_WIN
 #include <unistd.h>
@@ -222,7 +223,12 @@ KDiff3App::KDiff3App(QWidget* pParent, const QString& name, KDiff3Part* pKDiff3P
     m_sd2.setOptions(m_pOptions);
     m_sd3.setOptions(m_pOptions);
 
-    m_bAutoFlag = false; //disable --auto option git hard codes this unwanted flag.
+#ifdef ENABLE_AUTO
+    m_bAutoFlag = hasArgs && KDiff3Shell::getParser()->isSet("auto");
+#else
+    m_bAutoFlag = false;
+#endif
+
     m_bAutoMode = m_bAutoFlag || m_pOptions->m_bAutoSaveAndQuitOnMergeWithoutConflicts;
     if(hasArgs) {
         m_outputFilename = KDiff3Shell::getParser()->value("output");
@@ -269,7 +275,7 @@ KDiff3App::KDiff3App(QWidget* pParent, const QString& name, KDiff3Part* pKDiff3P
         //Set m_bDirCompare flag
         m_bDirCompare = FileAccess(m_sd1.getFilename()).isDir();
 
-        QStringList aliasList; //KDiff3Shell::getParser()->values( "fname" );
+        QStringList aliasList = KDiff3Shell::getParser()->values( "fname" );
         QStringList::Iterator ali = aliasList.begin();
 
         QString an1 = KDiff3Shell::getParser()->value("L1");
@@ -312,6 +318,9 @@ KDiff3App::KDiff3App(QWidget* pParent, const QString& name, KDiff3Part* pKDiff3P
     ///////////////////////////////////////////////////////////////////
     // call inits to invoke all other construction parts
     initActions(actionCollection());
+    //Warning: Call this before connecting KDiff3App::slotUpdateAvailabilities or calling KXMLGUIClient::setXMLFile
+    MergeResultWindow::initActions(actionCollection());
+
     initStatusBar();
 
     m_pFindDialog = new FindDialog(this);
@@ -357,7 +366,7 @@ KDiff3App::KDiff3App(QWidget* pParent, const QString& name, KDiff3Part* pKDiff3P
     m_pDirectoryMergeInfo = new DirectoryMergeInfo(m_pDirectoryMergeSplitter);
     m_pDirectoryMergeWindow->setDirectoryMergeInfo(m_pDirectoryMergeInfo);
     m_pDirectoryMergeSplitter->addWidget(m_pDirectoryMergeInfo);
-
+    //Warning: Make sure DirectoryMergeWindow::initActions is called before this point or we can crash when selectionChanged is sent.
     connect(m_pDirectoryMergeWindow, &DirectoryMergeWindow::startDiffMerge, this, &KDiff3App::slotFileOpen2);
     connect(m_pDirectoryMergeWindow->selectionModel(), &QItemSelectionModel::selectionChanged, this, &KDiff3App::slotUpdateAvailabilities);
     connect(m_pDirectoryMergeWindow->selectionModel(), &QItemSelectionModel::currentChanged, this, &KDiff3App::slotUpdateAvailabilities);
@@ -376,29 +385,9 @@ KDiff3App::KDiff3App(QWidget* pParent, const QString& name, KDiff3Part* pKDiff3P
 
 void KDiff3App::completeInit(const QString& fn1, const QString& fn2, const QString& fn3)
 {
-    if(m_pKDiff3Shell != nullptr)
-    {
-        QSize size = m_pOptions->m_geometry;
-        QPoint pos = m_pOptions->m_position;
-        if(!size.isEmpty())
-        {
-            m_pKDiff3Shell->resize(size);
-
-            QRect visibleRect = QRect(pos, size) & QApplication::desktop()->rect();
-            if(visibleRect.width() > 100 && visibleRect.height() > 100)
-                m_pKDiff3Shell->move(pos);
-            if(!m_bAutoMode)
-            {
-                //Here we want the extra setup showMaximized does since the window has not be shown before
-                if(m_pOptions->m_bMaximised)
-                    m_pKDiff3Shell->showMaximized();// krazy:exclude=qmethods
-                else
-                    m_pKDiff3Shell->show();
-            }
-        }
-    }
     if(!fn1.isEmpty()) {
         m_sd1.setFilename(fn1);
+        m_bDirCompare = FileAccess(m_sd1.getFilename()).isDir();
     }
     if(!fn2.isEmpty()) {
         m_sd2.setFilename(fn2);
@@ -468,13 +457,22 @@ void KDiff3App::completeInit(const QString& fn1, const QString& fn2, const QStri
     }
     m_bAutoMode = false;
 
-    if(m_pKDiff3Shell)
+    KToggleFullScreenAction::setFullScreen(m_pKDiff3Shell, m_pOptions->m_bMaximised);
+
+    if(m_pKDiff3Shell && !m_pKDiff3Shell->isVisible())
     {
-        if(m_pOptions->m_bMaximised)
-            //We want showMaximized here as the window has never been shown.
-            m_pKDiff3Shell->showMaximized();// krazy:exclude=qmethods
-        else
-            m_pKDiff3Shell->show();
+        QSize size = m_pOptions->m_geometry;
+        QPoint pos = m_pOptions->m_position;
+        if(!size.isEmpty())
+        {
+            m_pKDiff3Shell->resize(size);
+
+            QRect visibleRect = QRect(pos, size) & QApplication::desktop()->rect();
+            if(visibleRect.width() > 100 && visibleRect.height() > 100)
+                m_pKDiff3Shell->move(pos);
+        }
+
+        m_pKDiff3Shell->show();
     }
 
     g_pProgressDialog->setStayHidden(false);
@@ -489,20 +487,21 @@ void KDiff3App::completeInit(const QString& fn1, const QString& fn2, const QStri
     if(!m_bDirCompare && m_pKDiff3Shell != nullptr)
     {
         bool bFileOpenError = false;
-        if((!m_sd1.isEmpty() && !m_sd1.hasData()) ||
-           (!m_sd2.isEmpty() && !m_sd2.hasData()) ||
-           (!m_sd3.isEmpty() && !m_sd3.hasData()))
+        if((!m_sd1.getErrors().isEmpty()) ||
+            (!m_sd2.getErrors().isEmpty()) ||
+            (!m_sd3.getErrors().isEmpty()))
         {
             QString text(i18n("Opening of these files failed:"));
             text += "\n\n";
-            if(!m_sd1.isEmpty() && !m_sd1.hasData())
-                text += " - " + m_sd1.getAliasName() + '\n';
-            if(!m_sd2.isEmpty() && !m_sd2.hasData())
-                text += " - " + m_sd2.getAliasName() + '\n';
-            if(!m_sd3.isEmpty() && !m_sd3.hasData())
-                text += " - " + m_sd3.getAliasName() + '\n';
+            if(!m_sd1.getErrors().isEmpty())
+                text += " - " + m_sd1.getAliasName() + '\n' + m_sd1.getErrors().join('\n') + '\n';
+            if(!m_sd2.getErrors().isEmpty())
+                text += " - " + m_sd2.getAliasName() + '\n' + m_sd2.getErrors().join('\n') + '\n';
+            if(!m_sd3.getErrors().isEmpty())
+                text += " - " + m_sd3.getAliasName() + '\n' + m_sd3.getErrors().join('\n') + '\n';
+
+            KMessageBox::sorry(this, text, i18n("File open error"));
 
-            KMessageBox::sorry(this, text, i18n("File Open Error"));
             bFileOpenError = true;
         }
 
@@ -793,7 +792,7 @@ void printDiffTextWindow(RLPainter& painter, const QRect& view, const QString& h
             QString s = headerText.mid(p);
             int i;
             for(i = 2; i < s.length(); ++i)
-                if(fm.width(s, i) > view.width())
+                if(Utils::getHorizontalAdvance(fm, s, i) > view.width())
                 {
                     --i;
                     break;
@@ -809,7 +808,7 @@ void printDiffTextWindow(RLPainter& painter, const QRect& view, const QString& h
 
     painter.translate(0, view.top());
     pDiffTextWindow->print(painter, view, line, linesPerPage);
-    painter.resetMatrix();
+    painter.resetTransform();
 }
 
 void KDiff3App::slotFilePrint()
@@ -834,6 +833,8 @@ void KDiff3App::slotFilePrint()
         m_pDiffTextWindow3->getSelectionRange(&firstSelectionD3LIdx, &lastSelectionD3LIdx, eD3LLineCoords);
     }
 
+    printDialog->setOption(QPrintDialog::PrintCurrentPage);
+
     if(firstSelectionD3LIdx >= 0) {
         printDialog->addEnabledOption(QPrintDialog::PrintSelection);
         //printer.setOptionEnabled(QPrinter::PrintSelection,true);
@@ -856,7 +857,7 @@ void KDiff3App::slotFilePrint()
     {
         slotStatusMsg(i18n("Printing..."));
         // create a painter to paint on the printer object
-        RLPainter painter(&printer, m_pOptions->m_bRightToLeftLanguage, width(), fontMetrics().width('W'));
+        RLPainter painter(&printer, m_pOptions->m_bRightToLeftLanguage, width(), Utils::getHorizontalAdvance(fontMetrics(),'W'));
 
         QPaintDevice* pPaintDevice = painter.device();
         int dpiy = pPaintDevice->logicalDpiY();
@@ -873,7 +874,7 @@ void KDiff3App::slotFilePrint()
         QString topLineText = i18n("Top line");
 
         //int headerWidth = fm.width( m_sd1.getAliasName() + ", "+topLineText+": 01234567" );
-        int headerLines = fm.width(m_sd1.getAliasName() + ", " + topLineText + ": 01234567") / columnWidth + 1;
+        int headerLines = Utils::getHorizontalAdvance(fm, m_sd1.getAliasName() + ", " + topLineText + ": 01234567") / columnWidth + 1;
 
         int headerMargin = headerLines * fm.height() + 3; // Text + one horizontal line
         int footerMargin = fm.height() + 3;
@@ -922,8 +923,12 @@ void KDiff3App::slotFilePrint()
                 pageList.push_back(i);
             }
         }
-
-        if(printer.printRange() == QPrinter::Selection)
+        else if(printer.printRange() == QPrinter::CurrentPage)
+        {
+            bPrintCurrentPage = true;
+            totalNofPages = 1;
+        }
+        else if(printer.printRange() == QPrinter::Selection)
         {
             bPrintSelection = true;
             if(firstSelectionD3LIdx >= 0)
@@ -952,8 +957,9 @@ void KDiff3App::slotFilePrint()
                     break;
                 page = *pageListIt;
                 line = (page - 1) * linesPerPage;
-                if(page == 10000) { // This means "Print the current page"
-                    bPrintCurrentPage = true;
+
+                if(bPrintCurrentPage)
+                {
                     // Detect the first visible line in the window.
                     line = m_pDiffTextWindow1->convertDiff3LineIdxToLine(currentFirstD3LIdx);
                 }
@@ -998,10 +1004,11 @@ void KDiff3App::slotFilePrint()
                 QString s = bPrintCurrentPage ? QString("")
                                               : QString::number(page) + '/' + QString::number(totalNofPages);
                 if(bPrintSelection) s += i18n(" (Selection)");
-                painter.drawText((view.right() - painter.fontMetrics().width(s)) / 2,
+                painter.drawText((view.right() - Utils::getHorizontalAdvance(painter.fontMetrics(), s)) / 2,
                                  view.bottom() + painter.fontMetrics().ascent() + 5, s);
 
                 bFirstPrintedPage = true;
+                if(bPrintCurrentPage) break;
             }
 
             if(bPrintSelection)
