@@ -21,6 +21,7 @@
 #include "optiondialog.h"
 #include "OptionItems.h"
 #include "ConfigValueMap.h"
+#include "ui_scroller.h"
 
 #include "diff.h"
 #include "smalldialogs.h"
@@ -44,7 +45,7 @@
 #include <QPlainTextEdit>
 #include <QPushButton>
 #include <QRadioButton>
-#include <QSettings>
+#include <QScrollArea>
 #include <QTextCodec>
 #include <QToolTip>
 
@@ -55,6 +56,7 @@
 #include <KMessageBox>
 #include <KSharedConfig>
 #include <KToolBar>
+
 #include <map>
 
 #define KDIFF3_CONFIG_GROUP "KDiff3 Options"
@@ -107,9 +109,9 @@ FontChooser::FontChooser(QWidget* pParent)
 
     QChar visualTab(0x2192);
     QChar visualSpace((ushort)0xb7);
-    m_pExampleTextEdit = new QPlainTextEdit(QString("The quick brown fox jumps over the river\n"
-                                                    "but the little red hen escapes with a shiver.\n"
-                                                    ":-)") +
+    m_pExampleTextEdit = new QPlainTextEdit(i18n("The quick brown fox jumps over the river\n"
+                                                 "but the little red hen escapes with a shiver.\n"
+                                                 ":-)") +
                                                 visualTab + visualSpace,
                                             this);
     m_pExampleTextEdit->setFont(m_font);
@@ -411,7 +413,9 @@ class OptionEncodingComboBox : public QComboBox, public OptionCodec
     {
         if(c != nullptr)
         {
-            QString codecName = QLatin1String(c->name());
+            QByteArray nameArray = c->name();
+            QLatin1String codecName = QLatin1String(nameArray);
+
             for(int i = 0; i < m_codecVec.size(); ++i)
             {
                 if(c == m_codecVec[i])
@@ -421,7 +425,7 @@ class OptionEncodingComboBox : public QComboBox, public OptionCodec
             // The m_codecVec.size will at this point return the value we need for the index.
             if(codecName == defaultName())
                 saveDefaultIndex(m_codecVec.size());
-            QString itemText = visibleCodecName.isEmpty() ? codecName : visibleCodecName + QStringLiteral(" (") + codecName + QStringLiteral(")");
+            QString itemText = visibleCodecName.isEmpty() ? codecName : visibleCodecName + QLatin1String(" (") + codecName + QLatin1String(")");
             addItem(itemText, (int)m_codecVec.size());
             m_codecVec.push_back(c);
         }
@@ -492,6 +496,7 @@ OptionDialog::OptionDialog(bool bShowDirMergeSettings, QWidget* parent) : KPageD
     setStandardButtons(QDialogButtonBox::Help | QDialogButtonBox::RestoreDefaults | QDialogButtonBox::Apply | QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
 
     setModal(true);
+    setMinimumSize(600, 500);
 
     //showButtonSeparator( true );
     //setHelp( "kdiff3/index.html", QString::null );
@@ -609,12 +614,20 @@ void OptionDialog::setupFontPage()
 
 void OptionDialog::setupColorPage()
 {
-    QFrame* page = new QFrame();
-    KPageWidgetItem* pageItem = new KPageWidgetItem(page, i18nc("Title for color settings page","Color"));
+    QScrollArea* pageFrame = new QScrollArea();
+    KPageWidgetItem* pageItem = new KPageWidgetItem(pageFrame, i18nc("Title for color settings page","Color"));
     pageItem->setHeader(i18n("Colors Settings"));
     pageItem->setIcon(QIcon::fromTheme(QStringLiteral("colormanagement")));
     addPage(pageItem);
 
+    QVBoxLayout* scrollLayout = new QVBoxLayout();
+    scrollLayout->setMargin(2);
+    scrollLayout->addWidget(pageFrame);
+
+    QScopedPointer<Ui::ScrollArea> scrollArea(new Ui::ScrollArea());
+    scrollArea->setupUi(pageFrame);
+    
+    QWidget* page = pageFrame->findChild<QWidget*>("contents");
     QVBoxLayout* topLayout = new QVBoxLayout(page);
     topLayout->setMargin(5);
 
@@ -769,12 +782,21 @@ void OptionDialog::setupColorPage()
 
 void OptionDialog::setupEditPage()
 {
-    QFrame* page = new QFrame();
-    KPageWidgetItem* pageItem = new KPageWidgetItem(page, i18n("Editor"));
+    QScrollArea* pageFrame = new QScrollArea();
+    KPageWidgetItem* pageItem = new KPageWidgetItem(pageFrame, i18n("Editor"));
     pageItem->setHeader(i18n("Editor Behavior"));
     pageItem->setIcon(QIcon::fromTheme(QStringLiteral("accessories-text-editor")));
     addPage(pageItem);
 
+    QVBoxLayout* scrollLayout = new QVBoxLayout();
+    scrollLayout->setMargin(2);
+    scrollLayout->addWidget(pageFrame);
+
+    QScopedPointer<Ui::ScrollArea> scrollArea(new Ui::ScrollArea());
+    scrollArea->setupUi(pageFrame);
+    
+    QWidget* page = pageFrame->findChild<QWidget*>("contents");
+
     QVBoxLayout* topLayout = new QVBoxLayout(page);
     topLayout->setMargin(5);
 
@@ -821,9 +843,9 @@ void OptionDialog::setupEditPage()
     OptionComboBox* pLineEndStyle = new OptionComboBox(eLineEndStyleAutoDetect, "LineEndStyle", (int*)&m_options.m_lineEndStyle, page);
     gbox->addWidget(pLineEndStyle, line, 1);
     addOptionItem(pLineEndStyle);
-    pLineEndStyle->insertItem(eLineEndStyleUnix, "Unix");
-    pLineEndStyle->insertItem(eLineEndStyleDos, "Dos/Windows");
-    pLineEndStyle->insertItem(eLineEndStyleAutoDetect, "Autodetect");
+    pLineEndStyle->insertItem(eLineEndStyleUnix, i18nc("Unix line ending", "Unix"));
+    pLineEndStyle->insertItem(eLineEndStyleDos, i18nc("Dos/Windows line ending", "Dos/Windows"));
+    pLineEndStyle->insertItem(eLineEndStyleAutoDetect, i18nc("Automatically detected line ending", "Autodetect"));
 
     label->setToolTip(i18n(
         "Sets the line endings for when an edited file is saved.\n"
@@ -835,12 +857,21 @@ void OptionDialog::setupEditPage()
 
 void OptionDialog::setupDiffPage()
 {
-    QFrame* page = new QFrame();
-    KPageWidgetItem* pageItem = new KPageWidgetItem(page, i18n("Diff"));
+    QScrollArea* pageFrame = new QScrollArea();
+    KPageWidgetItem* pageItem = new KPageWidgetItem(pageFrame, i18n("Diff"));
     pageItem->setHeader(i18n("Diff Settings"));
     pageItem->setIcon(QIcon::fromTheme(QStringLiteral("text-x-patch")));
     addPage(pageItem);
 
+    QVBoxLayout* scrollLayout = new QVBoxLayout();
+    scrollLayout->setMargin(2);
+    scrollLayout->addWidget(pageFrame);
+
+    QScopedPointer<Ui::ScrollArea> scrollArea(new Ui::ScrollArea());
+    scrollArea->setupUi(pageFrame);
+    
+    QWidget* page = pageFrame->findChild<QWidget*>("contents");
+
     QVBoxLayout* topLayout = new QVBoxLayout(page);
     topLayout->setMargin(5);
 
@@ -921,12 +952,21 @@ void OptionDialog::setupDiffPage()
 
 void OptionDialog::setupMergePage()
 {
-    QFrame* page = new QFrame();
-    KPageWidgetItem* pageItem = new KPageWidgetItem(page, i18n("Merge"));
+    QScrollArea* pageFrame = new QScrollArea();
+    KPageWidgetItem* pageItem = new KPageWidgetItem(pageFrame, i18nc("Settings page", "Merge"));
     pageItem->setHeader(i18n("Merge Settings"));
     pageItem->setIcon(QIcon::fromTheme(QStringLiteral("merge")));
     addPage(pageItem);
 
+    QVBoxLayout* scrollLayout = new QVBoxLayout();
+    scrollLayout->setMargin(2);
+    scrollLayout->addWidget(pageFrame);
+
+    QScopedPointer<Ui::ScrollArea> scrollArea(new Ui::ScrollArea());
+    scrollArea->setupUi(pageFrame);
+    
+    QWidget* page = pageFrame->findChild<QWidget*>("contents");
+
     QVBoxLayout* topLayout = new QVBoxLayout(page);
     topLayout->setMargin(5);
 
@@ -1120,12 +1160,20 @@ void OptionDialog::setupMergePage()
 
 void OptionDialog::setupDirectoryMergePage()
 {
-    QFrame* page = new QFrame();
-    KPageWidgetItem* pageItem = new KPageWidgetItem(page, i18n("Directory"));
+    QScrollArea*  pageFrame = new QScrollArea();
+    KPageWidgetItem* pageItem = new KPageWidgetItem(pageFrame, i18n("Directory"));
     pageItem->setHeader(i18n("Directory"));
     pageItem->setIcon(QIcon::fromTheme(QStringLiteral("inode-directory")));
     addPage(pageItem);
 
+    QVBoxLayout* scrollLayout = new QVBoxLayout();
+    scrollLayout->setMargin(2);
+    scrollLayout->addWidget(pageFrame);
+
+    QScopedPointer<Ui::ScrollArea> scrollArea(new Ui::ScrollArea());
+    scrollArea->setupUi(pageFrame);
+    
+    QWidget* page = pageFrame->findChild<QWidget*>("contents");
     QVBoxLayout* topLayout = new QVBoxLayout(page);
     topLayout->setMargin(5);
 
@@ -1378,12 +1426,21 @@ void OptionDialog::setupRegionalPage()
         new Utf8BOMCodec();
    */
 
-    QFrame* page = new QFrame();
-    KPageWidgetItem* pageItem = new KPageWidgetItem(page, i18n("Regional Settings"));
+    QScrollArea* pageFrame = new QScrollArea();
+    KPageWidgetItem* pageItem = new KPageWidgetItem(pageFrame, i18n("Regional Settings"));
     pageItem->setHeader(i18n("Regional Settings"));
     pageItem->setIcon(QIcon::fromTheme(QStringLiteral("preferences-desktop-locale")));
     addPage(pageItem);
 
+    QVBoxLayout* scrollLayout = new QVBoxLayout();
+    scrollLayout->setMargin(2);
+    scrollLayout->addWidget(pageFrame);
+
+    QScopedPointer<Ui::ScrollArea> scrollArea(new Ui::ScrollArea());
+    scrollArea->setupUi(pageFrame);
+    
+    QWidget* page = pageFrame->findChild<QWidget*>("contents");
+
     QVBoxLayout* topLayout = new QVBoxLayout(page);
     topLayout->setMargin(5);
 
@@ -1481,12 +1538,20 @@ void OptionDialog::setupRegionalPage()
 
 void OptionDialog::setupIntegrationPage()
 {
-    QFrame* page = new QFrame();
-    KPageWidgetItem* pageItem = new KPageWidgetItem(page, i18n("Integration"));
+    QScrollArea* pageFrame = new QScrollArea();
+    KPageWidgetItem* pageItem = new KPageWidgetItem(pageFrame, i18n("Integration"));
     pageItem->setHeader(i18n("Integration Settings"));
     pageItem->setIcon(QIcon::fromTheme(QStringLiteral("utilities-terminal")));
     addPage(pageItem);
 
+    QVBoxLayout* scrollLayout = new QVBoxLayout();
+    scrollLayout->setMargin(2);
+    scrollLayout->addWidget(pageFrame);
+
+    QScopedPointer<Ui::ScrollArea> scrollArea(new Ui::ScrollArea());
+    scrollArea->setupUi(pageFrame);
+    
+    QWidget* page = pageFrame->findChild<QWidget*>("contents");
     QVBoxLayout* topLayout = new QVBoxLayout(page);
     topLayout->setMargin(5);
 
@@ -1579,16 +1644,6 @@ void OptionDialog::slotApply()
     }
 
     emit applyDone();
-
-#ifdef Q_OS_WIN
-    QString locale = m_options.m_language;
-    if(locale == "Auto" || locale.isEmpty())
-        locale = QLocale::system().name().left(2);
-    int spacePos = locale.indexOf(' ');
-    if(spacePos > 0) locale = locale.left(spacePos);
-    QSettings settings(QLatin1String("HKEY_CURRENT_USER\\Software\\KDiff3\\diff-ext"), QSettings::NativeFormat);
-    settings.setValue(QLatin1String("Language"), locale);
-#endif
 }
 
 /** Set the default values in the widgets only, while the
