@@ -59,14 +59,12 @@ OpenDialog::OpenDialog(
     h->setColumnStretch(1, 10);
 
     QLabel* label = new QLabel(i18n("A (Base):"), this);
-    QUrl url = QUrl(n1);
-    //QUrl::isLocalFile returns false if the scheme is blank.
-    if(url.scheme().isEmpty()) url.setScheme("file");
 
     m_pLineA = new QComboBox();
+    m_pLineA->setAcceptDrops(true);
     m_pLineA->setEditable(true);
     m_pLineA->insertItems(0, m_pOptions->m_recentAFiles);
-    m_pLineA->setEditText(url.isLocalFile() ? n1 : url.toDisplayString());
+    m_pLineA->setEditText(n1);
     m_pLineA->setMinimumWidth(200);
     QPushButton* button = new QPushButton(QIcon::fromTheme("document-new"), i18n("File..."), this);
     connect(button, &QPushButton::clicked, this, &OpenDialog::selectFileA);
@@ -79,13 +77,12 @@ OpenDialog::OpenDialog(
     h->addWidget(button, 0, 2);
     h->addWidget(button2, 0, 3);
 
-    url.setUrl(n2);
-    if(url.scheme().isEmpty()) url.setScheme("file");
     label = new QLabel("B:", this);
     m_pLineB = new QComboBox();
+    m_pLineB->setAcceptDrops(true);
     m_pLineB->setEditable(true);
     m_pLineB->insertItems(0, m_pOptions->m_recentBFiles);
-    m_pLineB->setEditText(url.isLocalFile() ? n2 :url.toDisplayString());
+    m_pLineB->setEditText(n2);
     m_pLineB->setMinimumWidth(200);
     button = new QPushButton(QIcon::fromTheme("document-new"), i18n("File..."), this);
     connect(button, &QPushButton::clicked, this, &OpenDialog::selectFileB);
@@ -98,13 +95,12 @@ OpenDialog::OpenDialog(
     h->addWidget(button, 1, 2);
     h->addWidget(button2, 1, 3);
 
-    url.setUrl(n3);
-    if(url.scheme().isEmpty()) url.setScheme("file");
     label = new QLabel(i18n("C (Optional):"), this);
     m_pLineC = new QComboBox();
+    m_pLineC->setAcceptDrops(true);
     m_pLineC->setEditable(true);
     m_pLineC->insertItems(0, m_pOptions->m_recentCFiles);
-    m_pLineC->setEditText(url.isLocalFile() ? n3 :url.toDisplayString());
+    m_pLineC->setEditText(n3);
     m_pLineC->setMinimumWidth(200);
     button = new QPushButton(QIcon::fromTheme("document-new"), i18n("File..."), this);
     connect(button, &QPushButton::clicked, this, &OpenDialog::selectFileC);
@@ -142,14 +138,12 @@ OpenDialog::OpenDialog(
 
     hl->addStretch(2);
 
-    url.setUrl(outputName);
-    if(url.scheme().isEmpty()) url.setScheme("file");
-
     label = new QLabel(i18n("Output (optional):"), this);
     m_pLineOut = new QComboBox();
+    m_pLineOut->setAcceptDrops(true);
     m_pLineOut->setEditable(true);
     m_pLineOut->insertItems(0, m_pOptions->m_recentOutputFiles);
-    m_pLineOut->setEditText(url.isLocalFile() ? outputName : url.toDisplayString());
+    m_pLineOut->setEditText(outputName);
     m_pLineOut->setMinimumWidth(200);
     button = new QPushButton(QIcon::fromTheme("document-new"), i18n("File..."), this);
     connect(button, &QPushButton::clicked, this, &OpenDialog::selectOutputName);
@@ -241,8 +235,8 @@ void OpenDialog::selectURL(QComboBox* pLine, bool bDir, int i, bool bSave)
 
     currentUrl = QUrl::fromUserInput(current, QString(), QUrl::AssumeLocalFile);
     QUrl newURL = bDir ? QFileDialog::getExistingDirectoryUrl(this, i18n("Open Directory"), currentUrl)
-                       : bSave ? QFileDialog::getSaveFileUrl(this, i18n("Select Output File"), currentUrl, QLatin1Literal("all/allfiles (*)"))
-                               : QFileDialog::getOpenFileUrl(this, i18n("Open File"), currentUrl, QLatin1Literal("all/allfiles (*)"));
+                       : bSave ? QFileDialog::getSaveFileUrl(this, i18n("Select Output File"), currentUrl, QLatin1String("all/allfiles (*)"))
+                               : QFileDialog::getOpenFileUrl(this, i18n("Open File"), currentUrl, QLatin1String("all/allfiles (*)"));
     if(!newURL.isEmpty()) {
         /*
         Since we are selecting a directory open in the parent directory
